@@ -4,6 +4,8 @@
 mod acceptance;
 /// Helpers.
 mod common;
+/// Configuration discovery tests.
+mod config;
 /// "Big picture" end-to-end tests, i.e. tests that typically exercise
 /// more than one audit or complex CLI functionality.
 mod e2e;
